unit alg3;
interface

uses GCD, Strcts, Motzkin, GL;
type
  TPoint= object
    X: LongInt;
    Y: LongInt;
    procedure Assign(NewX, NewY: LongInt);
    procedure AssignVector(StartPoint, EndPoint: TPoint);
    procedure Plus(FirstPoint, SecondPoint: TPoint; Alpha: LongInt);
    procedure Mult(Alpha: LongInt);
    procedure Division(Alpha: LongInt);
  end;

  PThresholdFunction = ^TThresholdFunction;
  TThresholdFunction = object
    K1, K2: LongInt;
    ABCAreKnown: Boolean;
    A, B, C: LongInt;
    Cone, Ostow, Incidence: PLongMatrix;
    ExtremeZeros, ExtremeOnes: PLongMatrix;
    constructor Init;
    destructor Done;
  private
    procedure InitByABC(X, Y, A1, A2, A0: LongInt);{ ᯮ}
    procedure InitByCone(X, Y: LOngInt;
      ACone, AnOstow, AnIncidence, Ones, Zeros: PLongMatrix);
  end;

  POracle = ^TOracle;
  TOracle = object
    constructor Init;
    function Answer(Point: TPoint): Boolean; virtual;
  end;

  PABCOracle = ^TABCOracle;
  TABCOracle = object(TOracle)
    A, B, C: LongInt;
    constructor Init(A1, A2, A0: LongInt);
    function Answer(Point: TPoint): Boolean; virtual;
  end;

  PDialogOracle = ^TDialogOracle;
  TDialogOracle = object(TOracle)
    function Answer(Point: TPoint): Boolean; virtual;
  end;

  PText = ^Text;

  PDecipheringAlg = ^TDecipheringAlg;
  TDecipheringAlg = object
    CurCone, CurOstow, CurBasis, CurZeros, CurOnes, CurIncidence: PLongMatrix;
    QueriesList: PLongMatrix;
    K1, K2: LongInt;
    Oracle: POracle;
    TotalAppeals, TargetAppeals: Word;
    Func: PThresholdFunction;
    HistoryFile: PText;
    AlgTerminated: Boolean;

    constructor Init(X, Y: LongInt; AnOracle: POracle;
      AF: PThresholdFunction; F: PText);
    destructor Done; virtual;
    function Run: Boolean; virtual;
    function Query(Point: TPoint): boolean;
    function Appeal(Point: TPoint): boolean;
    procedure OnAppeal(Point: TPoint; TargetAppeal, Value: Boolean); virtual;
    procedure OnEnd; virtual;
  end;

  PAlg3 = ^TAlg3;
  TAlg3 = object(TDecipheringAlg)
    constructor Init(X, Y: LongInt; AnOracle: POracle;
      AF: PThresholdFunction; F: PText);
    function Run: Boolean; virtual;
    procedure Dihotomia(StartPoint, EndPoint: TPoint;
      var ResultStartPoint, ResultEndPoint: TPoint);
    procedure FindConer(R0, R1, U, V: TPoint; var Coner: TPoint);
    procedure MakeItParallehedron(var R0, R1, R2, R3: TPoint;
      var AlgDone: Boolean);
    procedure FirstStep(var R0, R1, R2, R3: TPoint; var AlgDone: Boolean);
    procedure Parallehedron(var R0, R1, R2, R3: TPoint; var AlgDone: Boolean);
      virtual;
  end;

  PGraphAlg = ^TGraphAlg;
  TGraphAlg = object(TAlg3)
    FirstParallehedron: boolean;
    constructor Init(X, Y: LongInt; AnOracle: POracle;
      AF: PThresholdFunction; F: PText);
    procedure OnEnd; virtual;
    procedure OnAppeal(Point: TPoint; TargetAppeal, Value: Boolean); virtual;
    procedure Parallehedron(var R0, R1, R2, R3: TPoint; var AlgDone: Boolean);
      virtual;
  end;

  PGame = ^TGame;
  TGame = object(TDecipheringAlg)
    TeachingSetLeft: Byte;
    TeachingSet0, TeachingSet1: PLongMatrix;

    constructor Init(X, Y: LongInt; AnOracle: PABCOracle;
      AF: PThresholdFunction; F: PText);
    destructor Done; virtual;
    function Run: Boolean; virtual;
    procedure OnAppeal(Point: TPoint; TargetAppeal, Value: Boolean); virtual;
    procedure doAppeal; virtual;
  end;

  TGraphGame = object(TGame)
    constructor Init(X, Y: LongInt; AnOracle: PABCOracle;
       AF: PThresholdFunction; F: PText);
    procedure OnEnd; virtual;
    procedure OnAppeal(Point: TPoint; TargetAppeal, Value: Boolean); virtual;
    procedure doAppeal; virtual;
  end;

procedure RandomCoefficients(K1, K2: LongInt; var A, B, C: LongInt);

implementation

function IsEqual(A, B: TPoint): Boolean;
begin
  IsEqual := (A.X = B.X) and (A.Y = B.Y)
end;

function Product(A, B: TPoint): LongInt;
begin
  Product := A.X * B.X + A.Y * B.Y
end;

function Distance(A, B: TPoint): LongInt;
var R: TPoint;
begin
  R.AssignVector(A, B);
  Distance := Product(R, R);
end;

function Det(A, B: TPoint): LongInt;
var D: LongInt;
begin
  D := A.X * B.Y - A.Y * B.X;
  Det := D;
end;

procedure TPoint.Assign(NewX, NewY: LongInt);
begin
  X := NewX;
  Y := NewY;
end;

procedure TPoint.AssignVector(StartPoint, EndPoint: TPoint);
begin
  Plus(EndPoint, StartPoint, -1);
end;

procedure TPoint.Plus(FirstPoint, SecondPoint: TPoint; Alpha: LongInt);
begin
  X := FirstPoint.X + SecondPoint.X * Alpha;
  Y := FirstPoint.Y + SecondPoint.Y * Alpha;
end;

procedure TPoint.Mult(Alpha: LongInt);
begin
  X := X * Alpha;
  Y := Y * Alpha;
end;

procedure TPoint.Division(Alpha: LongInt);
begin
  X := X div Alpha;
  Y := Y div Alpha;
end;

procedure VectorAssign(var Vector; A1, A2, A3, A4: LongInt);
var V: TLongArray absolute Vector;
begin
  V[1] := A1;
  V[2] := A2;
  V[3] := A3;
  V[4] := A4;
end;

constructor TThresholdFunction.Init;
begin
  ABCAreKnown := False;
  Cone := nil;
  Ostow := nil;
  Incidence := nil;
  ExtremeZeros := nil;
  ExtremeOnes := nil;
end;

procedure TThresholdFunction.InitByABC(X, Y, A1, A2, A0: LongInt);
begin {  ᯮ}
  K1 := X;
  K2 := Y;
  ABCAreKnown := true;
  A := A1;
  B := A2;
  C := A0;
  Cone := nil;
  Ostow := nil;
  Incidence := nil;
  ExtremeZeros := nil;
  ExtremeOnes := nil;
end;

procedure TThresholdFunction.InitByCone(X, Y: LOngInt;
  ACone, AnOstow, AnIncidence, Ones, Zeros: PLongMatrix);
var
  i, j: Integer;
  Row: Array[1..4] of LongInt;
  MinThresholdValue: LongInt;
  TempOstow, TempIncidence, TempBasis: PLongMatrix;
  Item: LongInt;
begin
  ABCAreKnown := false;
  K1 := X;
  K2 := Y;
  if Cone <> nil then Dispose(Cone, Done);
  if Ostow <> nil then Dispose(Ostow, Done);
  if Incidence <> nil then Dispose(Incidence, Done);
  if ExtremeZeros <> nil then Dispose(ExtremeZeros, Done);
  if ExtremeOnes <> nil then Dispose(ExtremeOnes, Done);
  Cone := ACone;
  Ostow := AnOstow;
  Incidence := AnIncidence;
  ExtremeZeros := Zeros;
  ExtremeOnes := Ones;

  New(TempOstow, Init(2, 2));
  New(TempIncidence, Init(2, 2));
  New(TempBasis, Init(2, 2));

  Item := 1;
  with ExtremeZeros^ do
    for i := 1 to M do
      PutItem(i, 3, Item);

  Motzkin.Ostow(ExtremeZeros, TempOstow, TempIncidence, TempBasis, 0);

  Motzkin.Ostow(TempOstow, ExtremeZeros, TempIncidence, TempBasis, 0);

  {Motzkin.Ostow(ExtremeOnes, TempOstow, TempIncidence, TempBasis, 0);
  Motzkin.Ostow(TempOstow, ExtremeOnes, TempIncidence, TempBasis, 0);}

  Dispose(TempOstow, Done);
  Dispose(TempIncidence, Done);
  Dispose(TempBasis, Done);

  MinThresholdValue := MaxLongInt;
  for i := 1 to Ostow^.GetM do
  begin
    Ostow^.GetTheRow(i, Row);
    if (Row[4] > 0) and (Abs(Row[3]) < MinThresholdValue) then
    begin
      MinThresholdValue := Abs(Row[3]);
      ABCAreKnown := True;
      A := Row[1];
      B := Row[2];
      C := Row[3];
    end;
  end;
end;

destructor TThresholdFunction.Done;
begin
  if Cone <> nil then Dispose(Cone, Done);
  if Ostow <> nil then Dispose(Ostow, Done);
  if Incidence <> nil then Dispose(Incidence, Done);
  if ExtremeZeros <> nil then Dispose(ExtremeZeros, Done);
  if ExtremeOnes <> nil then Dispose(ExtremeOnes, Done);
end;

constructor TOracle.Init;
begin
end;

function TOracle.Answer(Point: TPoint): Boolean;
begin
end;


constructor TABCOracle.Init(A1, A2, A0: LongInt);
begin
  A := A1;
  B := A2;
  C := A0;
end;

function TABCOracle.Answer(Point: TPoint): Boolean;
begin
  Answer := (A * Point.X + B * Point.Y <= C)
end;

constructor TDecipheringAlg.Init(X, Y: LongInt; AnOracle: POracle;
  AF: PThresholdFunction; F: PText);
var Vector: Array[1..4] of LongInt;
begin
  K1 := X;
  K2 := Y;
  Oracle := AnOracle;
  Func := AF;
  HistoryFile := F;
  TotalAppeals := 0;
  TargetAppeals := 0;
  AlgTerminated := False;
  New(CurCone, Init(2, 2));
  New(CurOstow, Init(2, 2));
  New(CurBasis, Init(2, 2));
  New(CurIncidence, Init(2, 2));
  New(CurZeros, Init(2, 2));
  New(CurOnes, Init(2, 2));
  New(QueriesList, Init(2, 2));
  CurZeros^.Resize(0, 3);
  CurOnes^.Resize(0, 3);
  QueriesList^.Resize(0, 3);
  VectorAssign(Vector, 0, 0, 0, 1);
  with CurCone^ do
  begin
    Resize(0, 4);
    InsTheRow(1, @Vector);
  end;
  Ostow(CurCone, CurOstow, CurIncidence, CurBasis, 0);
end;

destructor TDecipheringAlg.Done;
begin
end;

procedure TDecipheringAlg.OnEnd;
begin
  Ostow(CurOstow, CurCone, CurIncidence, CurBasis, 0);
  Func^.InitByCone(K1, K2, CurCone, CurOstow, CurIncidence, CurOnes, CurZeros);

  if HistoryFile <> nil then
  begin
    Writeln(HistoryFile^, '㭪: '+ Long2Str(Func^.A, 0)+
      'x +'+Long2Str(Func^.B, 0)+'y  '+Long2Str(Func^.C, 0));
    Writeln(HistoryFile^, '᫮ ᮢ: '+
      Long2Str(TotalAppeals, 0));
    Writeln(HistoryFile^, '  `'': '+
      Long2Str(TargetAppeals, 0));
    Writeln(HistoryFile^, '');
    Writeln(HistoryFile^, '      x      y     f(x,y)');
    Writeln(HistoryFile^, '');
    QueriesList^.Print(HistoryFile^, 7);
  end;
  Dispose(QueriesList, Done);
  Dispose(CurBasis, Done);
end;

function TDecipheringAlg.Run: Boolean;
begin
  Run := not AlgTerminated;
end;

function TDecipheringAlg.Query(Point: TPoint): Boolean;
begin
  Query := Oracle^.Answer(Point);
end;


function TDecipheringAlg.Appeal(Point: TPoint): Boolean;
var Vector: Array[1..4] of LongInt;
begin
  Inc(TotalAppeals);
  VectorAssign(Vector, -Point.X, -Point.Y, 1, 0);
  If VectorInCone(Vector, CurOstow, CurBasis) then
  begin
    Appeal := True;
    OnAppeal(Point, False, True);
  end
  else
  begin
    VectorAssign(Vector, Point.X, Point.Y, -1, -1);
    if VectorInCone(Vector, CurOstow, CurBasis) then
    begin
      Appeal := False;
      OnAppeal(Point, False, False);
    end
    else { ``⢥'' }
    begin
      Inc(TargetAppeals);
      if Query(Point) then
      begin
        Appeal := True;
        VectorAssign(Vector, -Point.X, -Point.Y, 1, 0);
        CurCone^.InsTheRow(CurCone^.GetM + 1, @Vector);
        Ostow(CurCone, CurOstow, CurIncidence, CurBasis, 0);
        VectorAssign(Vector, Point.X, Point.Y, 0, 0);
        CurZeros^.InsTheRow(CurZeros^.GetM + 1, @Vector);
        QueriesList^.InsTheRow(QueriesList^.GetM + 1, @Vector);
        OnAppeal(Point, True, True);
      end
      else
      begin
        Appeal := False;
        VectorAssign(Vector, Point.X, Point.Y, -1, -1);
        CurCone^.InsTheRow(CurCone^.GetM + 1, @Vector);
        Ostow(CurCone, CurOstow, CurIncidence, CurBasis, 0);
        VectorAssign(Vector, Point.X, Point.Y, 1, 0);
        CurOnes^.InsTheRow(CurOnes^.GetM + 1, @Vector);
        QueriesList^.InsTheRow(QueriesList^.GetM + 1, @Vector);
        OnAppeal(Point, True, False);
      end;
    end;
  end;
end;

procedure TDecipheringAlg.OnAppeal(Point: TPoint;
  TargetAppeal, Value: Boolean);
begin
end;

constructor TAlg3.Init(X, Y: LongInt; AnOracle: POracle;
  AF: PThresholdFunction; F: PText);
begin
  inherited Init(X, Y, AnOracle, Af, F);
end;

procedure TAlg3.dihotomia(StartPoint, EndPoint: TPoint;
  var ResultStartPoint, ResultEndPoint: TPoint);
var
  Delta, TBegin, TEnd, TMed: LongInt;
  V, Point: TPoint;
begin
  V.AssignVector(StartPoint, EndPoint);
  Delta := LGCD(V.X, V.Y);
  TBegin := 0;
  TEnd := Delta;
  V.Division(Delta);
  while TEnd - TBegin > 1 do
  begin
    TMed := (TEnd + TBegin) div 2;
    Point.Plus(StartPoint, V, TMed);
    if Appeal(Point) then
      TBegin := TMed
    else
      TEnd := TMed;
  end;
  ResultStartPoint.Plus(StartPoint, V, TBegin);
  ResultEndPoint.Plus(StartPoint, V, TEnd);
end;

procedure TAlg3.FindConer(R0, R1, U, V: TPoint; var Coner: TPoint);
var
  Delta, Delta1, Delta2, T0, T1: LongInt;
  LeftPart: TPoint;
begin
  LeftPart.AssignVector(R1, R0);
  Delta := V.Y * U.X - V.X * U.Y;
  Delta1 := U.X * LeftPart.Y - U.Y * LeftPart.X;
  Delta2 := V.X * LeftPart.Y - V.Y * LeftPart.X;
  T0 := Delta2 div Delta;
  Coner.Plus(R0, U, T0);
end;

procedure TAlg3.MakeItParallehedron(var R0, R1, R2, R3: TPoint;
  var AlgDone: Boolean);
var
  U, V: TPoint;
  Coner: TPoint;
  T: LongInt;
begin
  U.AssignVector(R0, R1);
  V.AssignVector(R3, R2);
  AlgDone := False;
  if not IsEqual(U, V) then
  begin
    FindConer(R0, R3, U, V, Coner);
    if Distance(R0, Coner) > Distance(R3, Coner) then
      if Distance(R0, Coner) < Distance(R1, Coner) then
        R1.Plus(R0, V, 1)
      else
        R0.Plus(R1, V, -1)
    else if Distance(R0, Coner) < Distance(R3, Coner) then
      if Distance(R3, Coner) < Distance(R2, Coner) then
        R2.Plus(R3, U, 1)
      else
        R3.Plus(R2, U, -1)
    else
      AlgDone := True;
  end;
end;

procedure TAlg3.FirstStep(var R0, R1, R2, R3: TPoint; var AlgDone: Boolean);
var
  Point00, Point10, Point01, Point11: TPoint;
  F00, F10, F01, F11: Boolean;
  Flag: Boolean;
begin
  Point00.Assign(0, 0);
  F00 := Appeal(Point00);
  Point10.Assign(k1 - 1, 0);
  F10 := Appeal(Point10);
  Point01.Assign(0, k2 - 1);
  F01 := Appeal(Point01);
  Point11.Assign(k1 - 1, k2 - 1);
  F11 := Appeal(Point11);

  if F00 and F10 and F01 and F11 then
    AlgDone := true
  else if not(F00 or F10 or F01 or F11) then
    AlgDone := true
  else
  begin
    AlgDone := False;
    Flag := False;
    if (F00 and not F10) then
    begin
      Flag := True;
      Dihotomia(Point00, Point10, R0, R1);
    end;
    if (not F00 and F10) then
    begin
      Flag := True;
      Dihotomia(Point10, Point00, R0, R1);
    end;
    if (F00 and not F01) then
    begin
      if not Flag then Dihotomia(Point00, Point01, R0, R1)
      else Dihotomia(Point00, Point01, R3, R2);
      Flag := True;
    end;
    if (not F00 and F01) then
    begin
      if not Flag then Dihotomia(Point01, Point00, R0, R1)
      else Dihotomia(Point01, Point00, R3, R2);
      Flag := True;
    end;
    if (F01 and not F11) then
    begin
      if not Flag then Dihotomia(Point01, Point11, R0, R1)
      else Dihotomia(Point01, Point11, R3, R2);
      Flag := True;
    end;
    if (not F01 and F11) then
    begin
      if not Flag then Dihotomia(Point11, Point01, R0, R1)
      else Dihotomia(Point11, Point01, R3, R2);
      Flag := True;
    end;
    if (F11 and not F10) then
      Dihotomia(Point11, Point10, R3, R2);
    if (not F11 and F10) then
      Dihotomia(Point10, Point11, R3, R2);
    MakeItParallehedron(R0, R1, R2, R3, AlgDone);
  end;
end;

procedure FindSecondVectorInBasis(U: TPoint; var V: TPoint);
{to make unimodular basis}
var
  A: array[1..3,1..2] of LongInt;
  R: LongInt;
  i: Integer;
begin
  A[1, 1] := U.X; A[1, 2] := U.Y;
  A[2, 1] := 1; A[2, 2] := 0;
  A[3, 1] := 0; A[3, 2] := 1;
  if A[1, 1] < 0 then for i := 1 to 3 do A[i,1] := -A[i,1]; {⮫ * (-1)}
  if A[1, 2] < 0 then for i := 1 to 3 do A[i,2] := -A[i,2]; {⮫ * (-1)}
  while A[1, 2] <> 0 do
  begin
    R := A[1, 1] div A[1, 2];
    for i := 1 to 3 do {1- ⮫  2- ⮫ * R}
      A[i, 1] := A[i, 1] - A[i, 2] * R;
    for i := 1 to 3 do {⠢塞 ⮫}
    begin
      R := A[i, 1];
      A[i, 1] := A[i, 2];
      A[i, 2] := R;
    end;
  end;
  V.X := -A[3, 1];
  V.Y := A[2, 1];
  if A[1, 1] < 0 then U.Mult(-1);
end;

function DivLow(A, B: LongInt): LongInt;
var R: LongInt;
begin
  R := A div B;
  if ((A>=0) and (B<0) or (a<0) and (B>0)) and (A mod B <> 0) then R := R - 1;
  DivLow := R;
end;

function DivUp(A, B: LongInt): LongInt;
var R: LongInt;
begin
  R := A div B;
  if ((A>=0) and (B>0) or (A<0) and (B<0)) and (A mod B <> 0) then R := R + 1;
  DivUp := R;
end;

function Sign(A: LongInt): ShortInt;
begin
  if A > 0 then Sign := 1 else if A < 0 then Sign := -1 else Sign := 0;
end;

procedure Intersection(A, B, C, U: TPoint; var IntPoint, ExtPoint: TPoint);
var
  T: LongInt;
  A0, A1, A2: TPoint;
begin
  A1 := U;
  A2.Plus(A, B, -1);
  A0.Plus(A, C, -1);
  T := DivLow(Det(A0, A2), Det(A1, A2));
  IntPoint.Plus(C, U, T);
  ExtPoint.Plus(IntPoint, U, 1);
end;


procedure TAlg3.Parallehedron(var R0, R1, R2, R3: TPoint;
  var AlgDone: Boolean);
var
  U, V, W, WFirstPoint, R31, FirstPoint, FirstPoint1, IntPoint, ExtPoint,
  R0New, R1New, R2New, R3New: TPoint;
  Delta: LongInt;
begin
  U.AssignVector(R0, R1);
  FindSecondVectorInBasis(U, V);
  W.AssignVector(R0, R3);
  Delta := Sign(Det(U, W));
  if Delta = 0 then AlgDone := True
  else
  begin
    V.Mult(Delta); {⮡ V ᬮ५ ``'' ( ஭ W)}
    with R3 do
    begin
      R31.X := Delta * (V.Y * (X - R0.X) - V.X * (Y - R0.Y));
      R31.Y := Delta * (-U.Y * (X - R0.X) + U.X * (Y - R0.Y));
    end;
    with FirstPoint1 do
    begin
      Assign(DivUp(R31.X, R31.Y), 1);
      FirstPoint.X := R0.X + U.X * X + V.X * Y;
      FirstPoint.Y := R0.Y + U.Y * X + V.Y * Y;
    end;
    WFirstPoint.AssignVector(R0, FirstPoint);
    if Det(W, WFirstPoint) = 0 then AlgDone := True
    else
    begin
      Intersection(R1, R2, R0, WFirstPoint, IntPoint, ExtPoint);
      if not IsEqual(IntPoint, FirstPoint) then
      begin
        Dihotomia(R0, ExtPoint, R0New, R1New);
        WFirstPoint.Mult(-1);
        Intersection(R0, R3, R2, WFirstPoint, IntPoint, ExtPoint);
        Dihotomia(ExtPoint, R2, R3New, R2New);
      end
      else
      begin
        WFirstPoint.AssignVector(R1, FirstPoint);
        Intersection(R0, R3, R1, WFirstPoint, IntPoint, ExtPoint);
        Dihotomia(ExtPoint, R1, R0New, R1New);
        WFirstPoint.Mult(-1);
        Intersection(R1, R2, R3, WFirstPoint, IntPoint, ExtPoint);
        Dihotomia(R3, ExtPoint, R3New, R2New);
      end;
      R0 := R0New;
      R1 := R1New;
      R2 := R2New;
      R3 := R3New;
    end;
  end;
end;

function TAlg3.Run: Boolean;
var
  R0, R1, R2, R3: TPoint;
  AlgDone: Boolean;
begin
  FirstStep(R0, R1, R2, R3, AlgDone);
  while (not AlgDone) and (not AlgTerminated) do
  begin
    Parallehedron(R0, R1, R2, R3, AlgDone);
  end;
  Run := not AlgTerminated;
  OnEnd;
end;

constructor TGraphAlg.Init(X, Y: LongInt; AnOracle: POracle;
  AF: PThresholdFunction; F: PText);
begin
  inherited Init(X, Y, AnOracle, AF, F);
  FirstParallehedron := True;
  GraphInit;
  Limits(0, 0, X - 1, Y - 1, true, true);
  Grid(0, 0, 5, 5);
  Move(X-1, 0); Draw(X-1, Y-1); Draw(0, Y - 1);
end;

procedure TGraphAlg.OnEnd;
var
  i: Integer;
  Row: Array [1..4] of LongInt;
begin
  MouseOff;
  Title('That''s all');
  TitleBottom('Press ''q''');
  Wait(['Q', 'q']);
  inherited OnEnd;
  with func^ do
    if ABCAreKnown then
    begin
      ThickLines;
      Color(LightCyan);
      if (A <> 0) and (B <> 0) then
        LineByEquation(A, B, C);
      Color(White);
      ThinLines;
      Title(Long2Str(A, 0)+'x + '+Long2Str(B, 0)+'y  '+Long2Str(C, 0));
    end;
  with func^.Cone^ do
  begin
    for i := 1 to GetM do
    begin
      GetTheRow(i, Row);
      if (Row[3] = 1) and (Row[4] = 0) then
      begin
        Color(LightBlue); Move(-Row[1], -Row[2]);
        Point(5, True); Color(White);
      end
      else if (Row[3] = -1) and (Row[4] = -1) then
      begin
        Color(LightGreen); Move(Row[1], Row[2]);
        Point(5, True); Color(White);
      end;
    end;
  end;
  TitleBottom('Total number of queries: '+Long2Str(TotalAppeals, 0)+
    '   Amount `exact'' queris: '+  Long2Str(TargetAppeals, 0)+'     Press ''q''');
  Wait(['q', 'Q']);
  GraphDone;
end;


procedure TGraphAlg.OnAppeal(Point: TPoint; TargetAppeal, Value: Boolean);
begin
  Move(Point.X, Point.Y);
  if TargetAppeal then gl.Point(5, Value);
end;

procedure TGraphAlg.Parallehedron(var R0, R1, R2, R3: TPoint;
  var AlgDone: Boolean);
begin
  if FirstParallehedron then
  begin
    Move(R0.X, R0.Y);
    Draw(R1.X, R1.Y);
    Draw(R2.X, R2.Y);
    Draw(R3.X, R3.Y);
    Draw(R0.X, R0.Y);
    FirstParallehedron := False;
  end;
  inherited Parallehedron(R0, R1, R2, R3, AlgDone);
end;

function TDialogOracle.Answer(Point: TPoint): Boolean;
var Event: TEvent;
begin
  Title('Enter the value (0, 1) of the function in the point');
  Move(Point.X, Point.Y);
  Color(LightBlue); gl.Point(5, true);
  while true do
  begin
    WaitEvent(Event);
    with Event do
      if (What = KeyDown) and (Key in ['y','Y','0']) or
        (What = MouseDown) then Answer := True
      else if (What = KeyDown) and (Key in ['n','N','1']) or
        (What = RightMouseDown) then Answer := False
      else
        continue;
    break;
  end;
  Color(Black); gl.Point(5, true); Color(White);
end;

constructor TGame.Init(X, Y: LongInt; AnOracle: PABCOracle;
  AF: PThresholdFunction; F: PText);
var
  FunctionAux: PThresholdFunction;
  Alg: TAlg3;
  i: Integer;
  Row: Array [1..4] of LongInt;
begin
  inherited Init(X, Y, AnOracle, Af, F);

  {஢뢠 ᢮ ⬮  饬 魮
  ࠧ饣 ⢠ (TeachingSetLeft)}
  New(FunctionAux, Init);
  Alg.Init(X, Y, AnOracle, FunctionAux, nil);
  Alg.Run;
  Alg.Done;
  TeachingSetLeft := 0;
  New(TeachingSet0, Init(2, 2));
  TeachingSet0^.Resize(0, 2);
  New(TeachingSet1, Init(2, 2));
  TeachingSet1^.Resize(0, 2);
  with FunctionAux^.Cone^ do
    for i := 1 to GetM do
    begin
      GetTheRow(i, Row);
      if (Row[3] = 1) and (Row[4] = 0) then
      begin
        Row[1] := -Row[1];
        Row[2] := -Row[2];
        TeachingSet0^.InsTheRow(TeachingSet0^.M+1, @Row);
        Inc(TeachingSetLeft);
      end
      else if (Row[3] = -1) and (Row[4] = -1) then
      begin
        TeachingSet0^.InsTheRow(TeachingSet1^.M+1, @Row);
        Inc(TeachingSetLeft);
      end;
    end;
  Dispose(FunctionAux, Done);
end;

destructor TGame.Done;
begin
  Dispose(TeachingSet0, Done);
  Dispose(TeachingSet1, Done);
  Inherited Done;
end;

function PointInMatrix(Point: TPoint; Matrix: PLongMatrix): Boolean;
var
  i: Integer;
  Item1, Item2: LongInt;
begin
  with Matrix^ do
    for i := 1 to GetM do
    begin
      GetItem(i, 1, Item1);
      GetItem(i, 2, Item2);
      if (Item1 = Point.X) and (Item2 = Point.Y) then
      begin
        PointInMatrix := true;
        Exit;
      end;
    end;
  PointInMatrix := false;
end;

procedure TGame.OnAppeal(Point: TPoint; TargetAppeal, Value: Boolean);
begin
  if TargetAppeal and (PointInMatrix(Point, TeachingSet0) or
    PointInMatrix(Point, TeachingSet1)) then
    Dec(TeachingSetLeft);
end;

function TGame.Run: Boolean;
begin
  while (TeachingSetLeft > 0) and (not AlgTerminated) do
    doAppeal;
  Run := not AlgTerminated;
  OnEnd;
end;

procedure TGame.doAppeal;
begin
end;

constructor TGraphGame.Init(X, Y: LongInt; AnOracle: PABCOracle;
  AF: PThresholdFunction; F: PText);
begin
  inherited Init(X, Y, AnOracle, AF, F);
  GraphInit;
  Limits(0, 0, X - 1, Y - 1, true, true);
  Grid(0, 0, 5, 5);
  Move(X-1, 0); Draw(X-1, Y-1); Draw(0, Y - 1);
  Title('Choose any point by mouse');
  TitleBottom('For exit press ''q''');
  MouseOn;
end;

procedure TGraphGame.OnEnd;
var
  i: Integer;
  Row: Array [1..4] of LongInt;
begin
  MouseOff;
  Title('That''s all');
  TitleBottom('Press ''q''');
  Wait(['Q', 'q']);
  inherited OnEnd;
  with func^ do
    if ABCAreKnown then
    begin
      ThickLines;
      Color(LightCyan);
      if (A <> 0) and (B <> 0) then
        LineByEquation(A, B, C);
      Color(White);
      ThinLines;
      Title(Long2Str(A, 0)+'x + '+Long2Str(B, 0)+'y  '+Long2Str(C, 0));
    end;
  with func^.Cone^ do
  begin
    for i := 1 to GetM do
    begin
      GetTheRow(i, Row);
      if (Row[3] = 1) and (Row[4] = 0) then
      begin
        Color(LightBlue); Move(-Row[1], -Row[2]);
        Point(5, True); Color(White);
      end
      else if (Row[3] = -1) and (Row[4] = -1) then
      begin
        Color(LightGreen); Move(Row[1], Row[2]);
        Point(5, True); Color(White);
      end;
    end;
  end;
  TitleBottom('Total amount of queries: '+Long2Str(TotalAppeals, 0)+
    '   Amount of `exact'' queris '+
        Long2Str(TargetAppeals, 0)+'     Press ''q''');
  Wait(['q', 'Q']);
  GraphDone;
end;

procedure TGraphGame.OnAppeal(Point: TPoint; TargetAppeal, Value: Boolean);
begin
  Inherited OnAppeal(Point, TargetAppeal, Value);
  MouseOff;
  Move(Point.X, Point.Y);
  gl.Point(5, Value);
  if TargetAppeal then
    Title('f('+Long2Str(Point.X, 0)+','+Long2Str(Point.Y, 0)+')='+
      Boolean2Str(Value))
  else
    Title('Unnessasery query');
  MouseOn;
end;

procedure TGraphGame.doAppeal;
var
  PredExist: Boolean;
  Point: TPoint;
  Event: TEvent;
begin
  PredExist := False;
  while true do
  begin
    WaitEvent(Event);
    if (Event.What = KeyDown) and (Event.Key in ['q', 'Q']) then
    begin
      AlgTerminated := True;
      break;
    end;
    if (Event.What = KeyDown) then continue;
    Point.X := Round(Event.X);
    Point.Y := Round(Event.Y);
    if (Point.X >= K1) or (Point.Y >= K2) or (Point.X < 0) or (Point.Y < 0)
      then continue;
    if (Event.What = RightMouseDown) then
    begin
      MouseOff;
      if PredExist then
      begin
        Draw(Point.X, Point.Y);
        PredExist := False;
      end
      else
      begin
        Move(Point.X, Point.Y);
        PredExist := True;
      end;
      MouseOn;
      continue;
    end;
    Appeal(Point);
    Break;
  end;
end;

procedure RandomCoefficients(K1, K2: LongInt; var A, B, C: LongInt);
begin
  Randomize;
  A := Random(K2);
  B := Random(K1);
  C := Random(K1*K2);
end;


end.
