unit Motzkin;
interface
uses Strcts;

function Ostow(A, F, Q, E: PLongMatrix; Modification: Byte): ShortInt;
  {Find extreme rays F and basis E of a cone Ax >= 0
   return also an incidence matrix Q = F * A(Transp.)}

function VectorInCone(var Vector; Cone, Equalities: PLongMatrix): Boolean;
  {true if Vector belongs to a cone woth extreme rays Cone and basis (?)}

procedure State(S: string; A: LongInt); far;
  {shows the current state of a Motzkin-Burger process:
  writeln(S, A)}

procedure NoState(S: string; A: LongInt); register;
  {shows nothing}

const ShowState: procedure (S: string; A: LongInt) = @NoState;
  {a procedural-type constant showing the current state
  of a Motzkin-Burger process}

implementation
uses Gauss, GCD;

procedure State(S: string; A: LongInt);
begin
  writeln(S, A);
end;

procedure NoState(S: string; A: LongInt);
begin

end;

{**************************}
{*                        *}
{*     function Ostow     *}
{*                        *}
{**************************}

function Ostow(A, F, Q, E: PLongMatrix; Modification: Byte): ShortInt;
  {Find extreme rays F and basis E of a cone Ax >= 0
   return also an incidence matrix Q = F * A(Transp.)}

var
  currentOst, currentColumn, NewColumn, N, M, R, ii, iitmp, j_m, j_p: Integer;
  Det, b, b_minus, b_plus, Delta, Alpha_plus, Alpha_minus: LongInt;
  J_minus, J_plus, CommonZero: PIntSet;

{**************************}
{*                        *}
{* function Modification1 *}
{*                        *}
{**************************}

function Modification1: Integer;
var
  MinColumn, jj: Integer;
  MinFun, CurFun: LongInt;

function Fun(j: Integer): LongInt;
var
  i: Integer;
  Plus, Minus: Integer;
    { ⢮ ⥫  ⥫ ⮢
      ᮮ⢭⢥  j- ⮫ }
  b: LongInt;
begin
  Plus := 0;
  Minus := 0;
  for i := 1 to currentOst do
  begin
    Q^.GetItem(i, j, b);
    if b > 0 then
      Inc(Plus);
    if b < 0 then
      Inc(Minus);
  end;
  Fun := Plus*Minus;
end;

begin
 { ⮫,   ந
   ᫠ ⥫  ⥫ ⮢ }
  MinColumn := currentColumn;
  MinFun := Fun(MinColumn);
  for jj := currentColumn + 1 to M do
  begin
    CurFun := Fun(jj);
    if CurFun < MinFun then
    begin
      MinColumn := jj;
      MInFun := CurFun;
    end;
  end;
  Modification1 := MinColumn;
end;

{**************************}
{*                        *}
{*     function ReRoof    *}
{*                        *}
{**************************}

function ReRoof(Row: Integer): boolean;
var
  j: Integer;
  b: LongInt;
begin
  CommonZero^.StartIterate;
  while CommonZero^.NextExist do
  begin
    j := CommonZero^.GetNextInteger;
    Q^.GetItem(Row, j, b);
    if b <> 0 then
    begin
      {  ப  ४뢠 }
      ReRoof := false;
      Exit;
    end;
  end;
  ReRoof := true;
end;

{**************************}
{*                        *}
{*    function Balance    *}
{*                        *}
{**************************}

function Balance(j_plus, j_minus: Integer): boolean;
var
  j, jtmp, Row: Integer;
  b, b1, b2: LongInt;
begin

  { ந ⢮ CommonZero }
  for j := 1 to currentColumn-1 do
  begin
    Q^.GetItem(j_plus, j, b1);
    Q^.GetItem(j_minus, j, b2);
    if (b1=0) and (b2=0) then
    begin
      jtmp := j;
      CommonZero^.Insert(jtmp);
    end;
  end;

  if CommonZero^.N < R - 2 then
  {   㫥 }
  begin
    Balance := false;
    CommonZero^.DisposeAll;
    Exit;
  end;

  { ४뢠  㫨 }
  for Row := 1 to currentOst do
    if (Row <> j_plus) and (Row <> j_minus) then
      If ReRoof(Row) then
      begin
        Balance := false;
        CommonZero^.DisposeAll;
        Exit;
      end;
  Balance := true;
  CommonZero^.DisposeAll;
end;

{**************************}
{*      ⥫ 㭪樨      *}
{*          Ostow         *}
{**************************}

begin
  case LongMatrixGauss(A, F, Q, E, det) of
    -2:
    begin
      Ostow := -2; { 墠⨫ }
      Exit;
    end;
    1: Ostow := 1; {  ஦}
    2: Ostow := 2; { ஦}
  end;

  J_Plus := New(PIntSet, Init(10, 10));
  J_Minus := New(PIntSet, Init(10, 10));
  CommonZero := New(PIntSet, Init(10, 10));

  if (J_Plus = nil) or (J_Minus = nil) or (CommonZero = nil) then
  begin
    Ostow := -2;
    if J_Plus <> nil then
      Dispose(J_Plus, Done);
    if J_Minus <> nil then
      Dispose(J_Minus, Done);
    if CommonZero <> nil then
      Dispose(CommonZero, Done);
    Exit;
  end;

  N := A^.N;
  M := A^.M;
  R := N - E^.GetM;
  currentColumn := R+1;
  while currentColumn <= A^.M do
    {    ࠢ⢠  A, 稭  (R+1)- }
  begin
      ShowState('currentColumn = ', currentColumn);
      {writeln('currentColumn = ', currentColumn);}
      {Q^.ToFile(Output);
      ReadKey;}
    currentOst := Q^.M;
    {  室 ᫥饥 ࠢ⢮
       ( ᨬ  Modification).
       ᫨ Modification=0,  ६  ᫥饥
       (currentColumn) }
    case Modification of
      0: NewColumn := currentColumn;
      1: NewColumn := Modification1;
    end;
    if currentColumn <> NewColumn then
    begin
      Q^.SwapCols(currentColumn, NewColumn);
      A^.SwapRows(currentColumn, NewColumn);
    end;
    {  㥬 ⢠ J_plus, J_minus }
    for ii := 1 to currentOst do
    begin
      iitmp := ii;
      Q^.GetItem(iitmp, currentColumn, b);
      if b < 0 then
        J_minus^.Insert(iitmp)
      else
        If b > 0 then
          J_plus^.Insert(iitmp);
    end;
    if J_minus^.N = currentOst then
    begin
      {  襭, ⫨筮  㫥 }
      Ostow := -3;
      Exit;
    end;
    if J_minus^.N = 0 then
    begin
      { ࠢ⢮-᫥⢨ }
      A^.DelRow(currentColumn);
      Q^.DelCol(currentColumn);
      Dec(M);
      J_minus^.DisposeAll;
      J_plus^.DisposeAll;
      Continue; { 饥 ࠢ⢮  A }
    end;
    J_minus^.StartIterate;
    while J_minus^.NextExist do
    begin
      j_m := J_minus^.GetNextInteger;
      J_plus^.StartIterate;
      while J_plus^.NextExist do
      begin
        j_p := J_plus^.GetNextInteger;
        if Balance(j_p, j_m) then
        begin
          Q^.GetItem(j_m, currentColumn, b_minus);
          Q^.GetItem(j_p, currentColumn, b_plus);
          Delta := lGCD(b_minus, b_plus);
          Alpha_plus := -b_minus div Delta;
          Alpha_minus := b_plus div Delta;
          if (F^.AddRows(j_p, Alpha_plus, j_m, Alpha_minus) <> 1)
             or (Q^.AddRows(j_p, Alpha_plus, j_m, Alpha_minus) <> 1) then
          begin
            Ostow := -2;
            Exit;
          end;
          Delta := lGCD(vGCD(F^.GetRow(F^.M), N), vGCD(Q^.GetRow(Q^.M), M));
          F^.DivRow( F^.M, Delta);
          Q^.DivRow( Q^.M, Delta);
        end;
      end; { Iterate J_plus }
    end; { Iterate J_minus }
    J_minus^.StartIterate;
    while J_minus^.NextExist do
    begin
      j_m := J_minus^.GetNextInteger;
      F^.DisposeRow(j_m);
      Q^.DisposeRow(j_m);
    end;
    F^.DelNilRows;
    Q^.DelNilRows;
    J_minus^.DisposeAll;
    J_plus^.DisposeAll;
      {writeln('currentColumn = ', currentColumn);
      Q^.ToFile(Output);
      ReadKey;}
    Inc(currentColumn);
  end;
  Dispose(J_minus, Done);
  Dispose(J_plus, Done);
  Dispose(CommonZero, Done);
end;

{**************************}
{*                        *}
{* function VectorInCone  *}
{*                        *}
{**************************}

function VectorInCone(var Vector; Cone, Equalities: PLongMatrix): Boolean;
var
  V: TLongArray absolute Vector;
  Item, S: LongInt;
  Dimension, Rows, I, J: Integer;
begin
  VectorInCone := false;
  with Cone^ do
  begin
    Rows := GetM;
    Dimension := GetN;
    for i := 1 to Rows do
    begin
      S := 0;
      for j := 1 to Dimension do
      begin
        GetItem(i, j, Item);
        S := S + Item * V[j];
      end;
      if S < 0 then Exit;
    end;
  end;
  with Equalities^ do
  begin
    Rows := GetM;
    for i := 1 to Rows do
    begin
      S := 0;
      for j := 1 to Dimension do
      begin
        GetItem(i, j, Item);
        S := S + Item * V[j];
      end;
      if S <> 0 then Exit;
    end;
  end;
  VectorInCone := true;
end;


end.


