unit Strcts;
interface
uses Objects;

type
  PIntArray = ^TIntArray;
  TIntArray = Array[1..$FFFF div SizeOf(Integer)] of Integer;

  PLongArray = ^TLongArray;
  TLongArray = Array[1..$FFFF div SizeOf(LongInt)] of LongInt;

  PPointerArray = ^TPointerArray;
  TPointerArray = Array[1..$FFFF div SizeOf(Pointer)] of Pointer;

  PByteArray = ^TByteArray;
  TByteArray = array[1..$FFFF div SizeOf(Byte)] of Byte;

  TMatrix = object(TObject)
    N, M: Integer;
    ItemSize: byte;
    constructor Init;
    constructor Load(var S: TStream);
    procedure Store(var S: TStream);
    function GetN: integer; virtual;
    function GetM: integer; virtual;
    procedure GetItem(i, j: integer; var Item); virtual;
    procedure PutItem(i, j: integer; var Item); virtual;
    function InsCol(j: integer): ShortInt; virtual;
    function InsRow(i: integer): ShortInt; virtual;
    procedure DelCol(j: integer); virtual;
    procedure DelRow(i: integer); virtual;
    function FromArray: ShortInt; virtual;
    function FromFile(var F: Text): ShortInt; virtual;
    procedure ToArray; virtual;
    procedure ToFile(var F: Text); virtual;
    function SwapRows(i1, i2: integer): ShortInt; virtual;
    function MultRow(i: integer; var Alpha): ShortInt; virtual;
    function AddRow(i1, i2: integer; var Alpha): ShortInt; virtual;
      {  ப i1 ਡ i2, 㬭  Alpha}
    function AddRows(i1: integer; var Alpha1; i2: integer; var Alpha2):
      ShortInt; virtual;
     { ⠢  ப, ࠢ [i1]*Alpha1+[i2]*Alpha2 }
    function DivRow(i: integer; var Alpha): ShortInt; virtual;
    function SwapCols(j1, j2: integer): ShortInt; virtual;
    function MultCol(j: integer; var Alpha): ShortInt; virtual;
    function AddCol(j1, j2: integer; var Alpha): ShortInt; virtual;
      {  ⮫ j1 ਡ j2, 㬭  Alpha}
    function AddCols(j1: integer; var Alpha1; j2: integer; var Alpha2):
      ShortInt; virtual;
     { ⠢  ⮫, ࠢ [j1]*Alpha1+[j2]*Alpha2 }
    function DivCol(j: integer; var Alpha): ShortInt; virtual;
    procedure Error; virtual;
  end;

  TCollectionMatrix = object(TMatrix)
    MDelta: Integer;
    NDelta: Integer;
    MLimit: Integer;
    NLimit: Integer;
    Data: PPointerArray;
    constructor Init(AMDelta, ANDelta: Integer);
    destructor Done; virtual;
    constructor Load(var S: TStream);
    procedure Store(var S: TStream); virtual;
    procedure LoadItem(var S: TStream; var Item); virtual;
    procedure StoreItem(var S: TStream; var Item); virtual;
    function ReSize(AM, AN: Integer): ShortInt;
    function GetN: integer; virtual;
    function GetM: integer; virtual;
    procedure GetItem(i, j: integer; var Item); virtual;
    procedure PutItem(i, j: integer; var Item); virtual;
    function InsCol(j: integer): ShortInt; virtual;
    function InsRow(i: integer): ShortInt; virtual;
    procedure DelCol(j: integer); virtual;
    procedure DelRow(i: integer); virtual;
    procedure DisposeRow(i: integer); virtual;
    procedure DelNilRows; virtual;
    procedure DisposeItem(i, j: Integer); virtual;

    function GetRow(i: integer): pointer; virtual;
    procedure GetTheRow(i: integer; var Row); virtual;
    function InsThisRow(i: integer; P: Pointer): ShortInt; virtual;
    function InsTheRow(i: integer; P: Pointer): ShortInt; virtual;
    function TakeRow(i: Integer): Pointer;

    function SwapRows(i1, i2: integer): ShortInt; virtual;
 end;

  PLongMatrix = ^TLongMatrix;
  TLongMatrix = object(TCollectionMatrix)
    constructor Init(AMDelta, ANDelta: Integer);
    function MultRow(i: integer; var Alpha): ShortInt; virtual;
    function AddRow(i1, i2: integer; var Alpha): ShortInt; virtual;
      { ப i1 ਡ i2, 㬭  Alpha}
    function DivRow(i: integer; var Alpha): ShortInt; virtual;
    function DivRowGCD(i: integer): ShortInt; virtual;
    function MultCol(j: integer; var Alpha): ShortInt; virtual;
    function AddCol(j1, j2: integer; var Alpha): ShortInt; virtual;
    function DivCol(j: integer; var Alpha): ShortInt; virtual;
    function DivColGCD(j: integer): ShortInt; virtual;
    {function FromArray: ShortInt; virtual;}
    function FromFile(var F: text): ShortInt; virtual;
    {procedure ToArray; virtual;}
    procedure ToFile(var F: Text); virtual;
    procedure Print(var F: Text; Format: byte);
  end;

  PSet = ^TSet;
  TSet = object (TObject)
    N: integer;
    Limit: Integer;
    Data: PByteArray;
    Delta: Integer;
    ItemSize: Byte;
    constructor Init(ALimit, ADelta: Integer; AItemSize: Byte);
    constructor Load(var S: TStream);
    destructor Done; virtual;
    procedure Store(var S: TStream); virtual;
    function Insert(var Item): ShortInt;
    function NextExist: boolean;
    procedure StartIterate;
    procedure DisposeCurrent;
    procedure GetNext(var Item);
    procedure DisposeAll;
  private
    Iterate: Integer;
  end;

  PIntSet = ^TIntSet;
  TIntSet = object(TSet)
    constructor Init(ALimit, ADelta: Integer);
    function GetNextInteger: Integer;
  end;

  (*function TLongMatrix.Print(var F: Text): ShortInt;
var
  Item: LongInt;
  i, j: Integer;
  Format: PByteArray;
  S: String;
  Len: Byte absolute S;
begin
{$I-}
  Print := -1; {ᯥ譮 믮}
  Format := MemAlloc(N*SizeOf(LongInt));
  if Format = nil then Exit;
  for j := 1 to N do
  begin
    Format^[j] := 0;
    for i := 1 to M do
    begin
      GetItem(i, j, Item);
      Str(Item, S);
      if Len >= Format^[j] then Format^[j] := Len + 1;
    end;
  end;
  for i := 1 to M do
  begin
    for j := 1 to N do
    begin
      GetItem(i, j, Item);
      Write(F, Item: Format^[j]);
    end;
    Writeln(F);
  end;
{$I+}
  FreeMem(Format, N*SizeOf(LongInt));
  if IOResult = 0 then Print := 1;
end;*)


function Print3LongMatrix(var F: Text; A, B, C: PLongMatrix):
  ShortInt;

const
  RMatrix: TStreamRec=(
    ObjType: 10300;
    VmtLink: Ofs(TypeOf(TMatrix)^);
    Load: @TMatrix.Load;
    Store: @TMatrix.Store);

  RCollectionMatrix: TStreamRec=(
    ObjType: 10301;
    VmtLink: Ofs(TypeOf(TCollectionMatrix)^);
    Load: @TCollectionMatrix.Load;
    Store: @TCollectionMatrix.Store);

  RLongMatrix: TStreamRec=(
    ObjType: 10302;
    VmtLink: Ofs(TypeOf(TLongMatrix)^);
    Load: @TLongMatrix.Load;
    Store: @TLongMatrix.Store);

  RSet: TStreamRec=(
    ObjType: 10350;
    VmtLink: Ofs(TypeOf(TSet)^);
    Load: @TSet.Load;
    Store: @TSet.Store);

  RIntSet: TStreamRec=(
    ObjType: 10351;
    VmtLink: Ofs(TypeOf(TIntSet)^);
    Load: @TIntSet.Load;
    Store: @TIntSet.Store);

procedure RegisterStrcts;

implementation
uses SystemP, GCD, Memory;

constructor TMatrix.Init;
begin
end;

constructor TMatrix.Load(var S: TStream);
begin
  inherited Init;
  S.Read(N, SizeOf(N) + SizeOf(M) + SizeOf(ItemSize));
end;

procedure TMatrix.Store(var S: TStream);
begin
  S.Write(N, SizeOf(N) + SizeOf(M) + SizeOf(ItemSize));
end;

function TMatrix.GetN: integer;
begin
end;

function TMatrix.GetM: integer;
begin
end;

procedure TMatrix.GetItem(i, j: integer; var Item);
begin
end;

procedure TMatrix.PutItem(i, j: integer; var Item);
begin
end;

function TMatrix.InsRow(i: integer): ShortInt;
begin
end;

function TMatrix.InsCol(j: integer): ShortInt;
begin
end;

procedure TMatrix.DelCol(j: integer);
begin
end;

procedure TMatrix.DelRow(i: integer);
begin
end;

function TMatrix.FromArray: ShortInt;
begin
end;

function TMatrix.FromFile(var F: Text): ShortInt;
begin
end;

procedure TMatrix.ToArray;
begin
end;

procedure TMatrix.ToFile(var F: Text);
begin
end;

function TMatrix.SwapRows(i1, i2: integer): ShortInt;
begin
end;

function TMatrix.MultRow(i: integer; var Alpha): ShortInt;
begin
end;

function TMatrix.AddRow(i1, i2: integer; var Alpha): ShortInt;
  {  ப i1 ਡ i2, 㬭  Alpha}
begin
end;

function TMatrix.AddRows(i1: integer; var Alpha1; i2: integer; var Alpha2):
  ShortInt;
  { ⠢  ப, ࠢ [i1]*Alpha1+[i2]*Alpha2 }
var
  b: PByteArray;
  j: Integer;
begin
  if InsRow(M+1) <> 1 then
  begin
    AddRows := -1;
    Exit;
  end;
  b := MemAlloc(ItemSize);
  if b=nil then
  begin
    AddRows := -1;
    Exit;
  end;
  for j := 1 to ItemSize do
    b^[j] := 0;
  for j := 1 to N do
    PutItem(M, j, b^); { 㫨  ப }
  if (AddRow(M, i1, Alpha1)<>1) or (AddRow(M, i2, Alpha2)<>1) then
  begin
    DelRow(M);
    AddRows := -2;
  end
  else
    AddRows := 1;
  FreeMem(b, ItemSize);
end;

function TMatrix.DivRow(i: integer; var Alpha): ShortInt;
begin
end;

function TMatrix.MultCol(j: integer; var Alpha): ShortInt;
begin
end;

function TMatrix.AddCol(j1, j2: integer; var Alpha): ShortInt;
begin
end;

function TMatrix.DivCol(j: integer; var Alpha): ShortInt;
begin
end;

function TMatrix.AddCols(j1: integer; var Alpha1; j2: integer; var Alpha2):
  ShortInt;
var
  b: PByteArray; { ᨢ  㫥  ⮫ }
  i, j: Integer;
begin
  if InsCol(N+1) <> 1 then
  begin
    AddCols := -1;
    Exit;
  end;
  b := MemAlloc(ItemSize);
  if b=nil then
  begin
    AddCols := -1;
    Exit;
  end;
  for i := 1 to ItemSize do
    b^[i] := 0;
  for i := 1 to M do
    PutItem(i, N, b^); { 㫨  ப }
  if (AddCol(N, j1, Alpha1)<>1) or (AddCol(N, j2, Alpha2)<>1) then
  begin
    AddCols := -2;
    DelCol(N);
  end
  else
    AddCols := 1;
  FreeMem(b, ItemSize);
end;

function TMatrix.SwapCols(j1, j2: integer): ShortInt;
var
  i: Integer;
  P1, P2: Pointer;
begin
  P1 := MemAlloc(ItemSize);
  P2 := MemAlloc(ItemSize);
  If (P1=nil) or (P2=nil) then
  begin
    SwapCols := -1;
    if P1<>nil then FreeMem(P1, ItemSize);
    if P2<>nil then FreeMem(P2, ItemSize);
    Exit;
  end;
  for i:=1 to M do
  begin
    GetItem(i, j1, P1^);
    GetItem(i, j2, P2^);
    PutItem(i, j1, P2^);
    PutItem(i, j2, P1^);
  end;
  FreeMem(P1, ItemSize);
  FreeMem(P2, ItemSize);
  SwapCols := 1;
end;

procedure TMatrix.Error;
begin
  {Writeln('訡!');}
end;

constructor TCollectionMatrix.Init(AMDelta, ANDelta: Integer);
begin
  Inherited Init;
  MLimit := 0;
  NLimit := 0;
  MDelta := AMDelta;
  NDelta := ANDelta;
  Data := nil;
  {Data := MemAlloc(SizeOf(Pointer)*MLimit);
  if Data=Nil then
  begin
    Error;
    Fail;
    Exit;
  end;}
  M := 0;
  N := 0;
end;

constructor TCollectionMatrix.Load(var S: TStream);
var
  AM, AN: Integer;
  b: PByteArray;
  i, j: Integer;
begin
  inherited Load(S);
  S.Read(MDelta, SizeOf(MDelta) + SizeOf(NDelta) + SizeOf(MLimit) +
    SizeOf(NLimit));
  AM := M; AN := N;
  M := 0; N := 0;
  Data := MemAlloc(SizeOf(Pointer)*MLimit);
  if Data = nil then
  begin
    Error;
    Done;
    Fail;
    Exit;
  end;
  if Resize(AM, AN) <> 1 then
  begin
    Error;
    Done;
    Fail;
    Exit;
  end;
  b := GetMem(ItemSize);
  for i := 1 to M do
    for j := 1 to N do
    begin
      LoadItem(S, b^);
      PutItem(i, j, b^);
    end;
  FreeMem(b, ItemSize);
end;

procedure TCollectionMatrix.Store(var S: TStream);
var
  b: PByteArray;
  i, j: Integer;
begin
  inherited Store(S);
  S.Write(MDelta, SizeOf(MDelta) + SizeOf(NDelta) + SizeOf(MLimit) +
    SizeOf(NLimit));
  b := GetMem(ItemSize);
  for i := 1 to M do
    for j := 1 to N do
    begin
      GetItem(i, j, b^);
      StoreItem(S, b^);
    end;
  FreeMem(b, ItemSize);
end;

procedure TCollectionMatrix.LoadItem(var S: TStream; var Item);
begin
  S.Read(Item, ItemSize);
end;

procedure TCollectionMatrix.StoreItem(var S: TStream; var Item);
begin
  S.Write(Item, ItemSize);
end;

function TCollectionMatrix.ReSize(AM, AN: Integer): ShortInt;
var i: Integer;
begin
  ReSize := -1; {㤠筮 襭}
  while M > 0 do
    DelRow(M); {㤠  ப}
  if Data = nil then
  begin
    MLimit := AM;
    if MLimit = 0 then
      MLimit := 1;
    Data := MemAlloc(SizeOf(Pointer)*MLimit);
  end;
  if Data = nil then exit;
  NLimit := AN;
  M := 0;
  N := AN;
  for i := 1 to AM do
    if InsRow(i) <> 1 then Exit;
  ReSize := 1; {ᯥ譮 襭}
end;

destructor TCollectionMatrix.Done;
var i, j: integer;
begin
  for i := 1 to M do
    if Data^[i] <> nil then
    begin
      for j := 1 to N do DisposeItem(i, j);
      FreeMem(Data^[i], ItemSize*NLimit);
    end;
  if Data <> nil then
    FreeMem(Data, SizeOf(Pointer)*MLimit);
end;

function TCollectionMatrix.GetM: Integer;
begin
  GetM := M;
end;

function TCollectionMatrix.GetN: Integer;
begin
  GetN := N;
end;

procedure TCollectionMatrix.GetItem(i, j: integer; var Item);
var
  Source: pointer;
begin
  Source := Data^[i];
  MovePointer(Source, ItemSize*(j-1)); {।  j- }
  Move(Source^, Item, ItemSize);
end;

procedure TCollectionMatrix.PutItem(i, j: integer; var Item);
var
  Dest, Source: pointer;
begin
  Source := @Item;
  Dest := Data^[i];
  MovePointer(Dest, ItemSize*(j-1)); {।  j- }
  Move(Source^, Dest^, ItemSize);
end;

function TCollectionMatrix.InsCol(j: integer): ShortInt;
var
  i, ii: Integer;
  P, Dest, Source: pointer;
  StError: Pointer;
begin
  InsCol := 1; {ᯥ譮 믮}
  if N<NLimit then
  {墠⨫ १ࢨ஢ }
  begin
    for ii := 1 to M do
    begin
      Source := Data^[ii];
      MovePointer(Source, ItemSize*(j-1)); {।  j- }
      Dest := Data^[ii];
      MovePointer(Dest, ItemSize*j); {।  (j+1)- }
      Move(Source^, Dest^, ItemSize*(N-j+1));
    end;
    Inc(N);
  end
  else
  { 墠⨫ १ࢨ஢ }
  begin
    for i := 1 to M do
    begin
      P := MemAlloc((NLimit+NDelta)*ItemSize);
      if P=nil then
      begin
        { 㦭 ᭮ ᢮  -  ப,
             㦥 ।: }
        for ii := 1 to i-1 do
        begin
          Source := Data^[ii];
          P := MemAlloc(NLimit*ItemSize);
          if P = nil then
          begin
            InsCol := -2;
            {MessageBox('bad', nil, mfError+mfOkButton);
             ⠭  ()}
            Error;
            Exit;
          end;
          Dest := P;
          Move(Source^, Dest^, (j-1)*ItemSize);
          MovePointer(Source, j*ItemSize); {।  (j+1)- }
          MovePointer(Dest, (j-1)*ItemSize); {।  j- }
          Move(Source^, Dest^, (N-j+1)*ItemSize);
          FreeMem(Data^[ii], NLimit+NDelta);
          Data^[ii] := P;
        end;
        Error;
        InsCol := -1;
        Exit;
      end;
      Source := Data^[i];
      Dest := P;
      Move(Source^, Dest^, (j-1)*ItemSize);
      MovePointer(Source, (j-1)*ItemSize); {।  j- }
      MovePointer(Dest, j*ItemSize); {।  (j+1)- }
      Move(Source^, Dest^, (N-j+1)*ItemSize);
      FreeMem(Data^[i], NLimit*ItemSize);
      Data^[i] := P;
    end;
    Inc(NLimit, NDelta);
    Inc(N);
  end;
end;

function TCollectionMatrix.InsRow(i: Integer): ShortInt;
var
  Source, Dest, P: Pointer;
  NewData: PPointerArray;
begin
  InsRow := 1; {ᯥ譮 믮}
  P := MemAlloc(ItemSize*NLimit);
  if P=nil then
  begin
    Error;
    InsRow := -1;
    Exit;
  end;
  if InsThisRow(i, P) <> 1 then InsRow := -1;
end;

procedure TCollectionMatrix.DelCol(j: Integer);
var
  ii: Integer;
  Source, Dest: Pointer;
begin
  for ii := 1 to M do
  begin
    DisposeItem(ii, j);
    Source := Data^[ii];
    MovePointer(Source, j*ItemSize);  {।  (j+1)- }
    Dest := Data^[ii];
    MovePointer(Dest, (j-1)*ItemSize); {।  j- }
    Move(Source^, Dest^, ItemSize*(N-j));
  end;
  Dec(N);
end;

procedure TCollectionMatrix.DelRow(i: Integer);
var
  Source, Dest: Pointer;
  j: Integer;
begin
  if Data^[i] <> nil then
  begin
    for j := 1 to N do DisposeItem(i, j);
    FreeMem(Data^[i], ItemSize*NLimit);
  end;
  Source := @(Data^[i+1]);
  Dest := @(Data^[i]);
  Move(Source^, Dest^, ItemSize*(M-i));
  Dec(M);
end;

procedure TCollectionMatrix.DisposeRow(i: integer);
var j: Integer;
begin
  if Data^[i] <> nil then
  begin
    for j := 1 to N do DisposeItem(i, j);
    FreeMem(Data^[i], ItemSize * NLimit);
  end;
  Data^[i] := nil;
end;

procedure TCollectionMatrix.DelNilRows;
var i, NewI: Integer;
begin
  NewI := 0;
  for i := 1 to M do
  begin
    if Data^[i] <> nil then
    begin
      Inc(NewI);
      Data^[NewI] := Data^[i];
    end;
  end;
  M := NewI;
end;

function TCollectionMatrix.GetRow(i: Integer): Pointer;
begin
  GetRow := Data^[i];
end;

procedure TCollectionMatrix.GetTheRow(i: integer; var Row);
begin
  Move(GetRow(i)^, Row, ItemSize * N);
end;

function TCollectionMatrix.TakeRow(i: Integer): Pointer;
begin
  TakeRow := Data^[i];
  Data^[i] := nil;
  DelRow(i);
end;

function TCollectionMatrix.InsThisRow(i: Integer; P: pointer): ShortInt;
var
  Source, Dest: Pointer;
  NewData: PPointerArray;
begin
  InsThisRow := 1; {ᯥ譮 믮}
  if P=nil then
  begin
    InsThisRow := -1;
    Exit;
  end;
  if M<MLimit then
  { 墠⨫ १ࢨ஢  }
  begin
    Source := @(Data^[i]);
    Dest := @(Data^[i+1]);
    if M-i+1>0 then
      Move(Source^, Dest^, (M-i+1)*SizeOf(Pointer));
    Inc(M);
    Data^[i] := P;
  end
  else
  {  墠⨫ १ࢨ஢  }
  begin
    NewData := MemAlloc(SizeOf(Pointer)*(MLimit+MDelta));
    if NewData=nil then
    begin
      Error;
      InsThisRow := -1;
      Exit
    end;
    Source := Data;
    Dest := NewData;
    Move(Source^, Dest^, SizeOf(Pointer)*(i-1));
    MovePointer(Source, SizeOf(Pointer)*(i-1));
      {।  i- }
    MovePointer(Dest, SizeOf(Pointer)*i); {।  (i+1)- }
    Move(Source^, Dest^, SizeOf(Pointer)*(M-i+1));
    FreeMem(Data, SizeOf(Pointer)*MLimit);
    Data := NewData;
    Data^[i] := P;
    Inc(M);
    Inc(MLimit, MDelta);
  end;
end;

function TCollectionMatrix.InsTheRow(i: Integer; P: pointer): ShortInt;
var NewRow: Pointer;
begin
  If InsRow(i) = 1 then
  begin
    InsTheRow := 1;
    Move(P^, Data^[i]^, ItemSize * N);
  end
  else
    InsTheRow := -1;
end;

procedure TCollectionMatrix.DisposeItem(i, j: Integer);
begin
end;

function TCollectionMatrix.SwapRows(i1, i2: integer): ShortInt;
var P: Pointer;
begin
  P := Data^[i1];
  Data^[i1] := Data^[i2];
  Data^[i2] := P;
  SwapRows := 1;
end;

constructor TLongMatrix.Init(AMDelta, ANDelta: Integer);
begin
  Inherited Init(AMDelta, ANDelta);
  ItemSize := SizeOf(LongInt);
end;

function TLongMatrix.MultRow(i: integer; var Alpha): ShortInt;
var
  P: PLongArray;
  b: LongInt;
  j: Integer;
begin
  P := MemAlloc(SizeOf(LongInt)*NLimit);
  if P = nil then
    MultRow := -1
  else
  begin
    for j := 1 to N do
    begin
      GetItem(i, j, b);
      P^[j] := LongInt(Alpha)*b;
    end;
    FreeMem(Data^[i], SizeOf(LongInt)*NLimit);
    Data^[i] := P;
    MultRow := 1;
  end;
end;

function TLongMatrix.AddRow(i1, i2: integer; var Alpha): ShortInt;
  {  ப i1 ਡ i2, 㬭  Alpha}
var
  P: PLongArray;
  b1, b2: LongInt;
  j: Integer;
begin
  P := MemAlloc(SizeOf(LongInt)*NLimit);
  if P = nil then
    AddRow := -1
  else
  begin
    for j := 1 to N do
    begin
      GetItem(i1, j, b1);
      GetItem(i2, j, b2);
      P^[j] := b1+LongInt(Alpha)*b2; {!!!perepolnenie}
    end;
    FreeMem(Data^[i1], SizeOf(LongInt)*NLimit);
    Data^[i1] := P;
    AddRow := 1;
  end;
end;

function TLongMatrix.DivRow(i: integer; var Alpha): ShortInt;
var
  P: PLongArray;
  b: LongInt;
  j: Integer;
begin
  if LongInt(Alpha) = 0 then
  begin
    DivRow := -2;
    exit;
  end;
  P := MemAlloc(SizeOf(LongInt)*NLimit);
  if P = nil then
    DivRow := -1
  else
  begin
    for j := 1 to N do
    begin
      GetItem(i, j, b);
      if b mod LongInt(Alpha) <> 0 then
      begin
        FreeMem(P, SizeOf(LongInt)*NLimit);
        DivRow := -2;
        Exit;
      end;
      P^[j] := b div LongInt(Alpha);
    end;
    FreeMem(Data^[i], SizeOf(LongInt)*NLimit);
    Data^[i] := P;
    DivRow := 1;
  end;
end;

function TLongMatrix.DivRowGCD(i: integer): ShortInt;
var Alpha: LongInt;
begin
  Alpha := vGCD(Data^[i], N);
  {MessageBox('=%d', @Alpha, mfOkButton or mfInformation);}
  if Alpha<>0 then
    DivRow(i, Alpha);
  DivRowGCD := 1;
end;

function TLongMatrix.MultCol(j: integer; var Alpha): ShortInt;
var
  i: Integer;
  b: LongInt;
begin
  for i := 1 to M do
  begin
    GetItem(i, j, b);
    b := b*LongInt(Alpha);
    PutItem(i, j, b);
  end;
  MultCol := 1;
end;

function TLongMatrix.AddCol(j1, j2: integer; var Alpha): ShortInt;
var
  i: Integer;
  b1, b2: LongInt;
begin
  for i := 1 to M do
  begin
    GetItem(i, j1, b1);
    GetItem(i, j2, b2);
    Inc(b1, b2*LongInt(Alpha));
    PutItem(i, j1, b1);
  end;
  AddCol := 1;
end;

function TLongMatrix.DivCol(j: integer; var Alpha): ShortInt;
var
  P: PLongArray;
  b: LongInt;
  i: Integer;
begin
  if LongInt(Alpha) = 0 then
  begin
    DivCol := -2;
    exit;
  end;
  for i := 1 to M do
  begin
    GetItem(i, j, b);
    if b mod LongInt(Alpha) <> 0 then
    begin
      DivCol := -2;
      Exit;
    end;
    b := b div LongInt(Alpha);
    PutItem(i, j, b);
  end;
  DivCol := 1;
end;

function TLongMatrix.DivColGCD(j: integer): ShortInt;
var
  i: Integer;
  b, d: LongInt;
begin
  { 室  }
  GetItem(M, j, d);
  d := Abs(d);
  i := M-1;
  while (d<>1) and (i>0) do
  begin
    GetItem(i, j, b);
    d := lGCD(b, d);
    Dec(i);
  end;
  {MessageBox('=%d', @d, mfOkButton or mfInformation);}
  {  ⮫   }
  if d<>0 then
    DivCol(j, d);
  DivColGCD := 1;
end;

function TLongMatrix.FromFile(var F: Text): ShortInt;
var
  Item: LongInt;
  i, j: Integer;
begin
{$I-}
  FromFile := 1; {ᯥ譮 믮}
  for i := 1 to M do
    for j := 1 to N do
    begin
      Read(F, Item);
      PutItem(i, j, Item);
    end;
{$I+}
  if IOResult <> 0 then FromFile := -1;
end;

procedure TLongMatrix.ToFile(var F: Text);
var
  Item: LongInt;
  i, j: Integer;
begin
  for i := 1 to M do
  begin
    for j := 1 to N do
    begin
      GetItem(i, j, Item);
      Write(F, Item{: 12},'  ');
    end;
    Writeln(F);
  end;
end;

procedure TLongMatrix.Print(var F: Text; Format: byte);
var
  Item: LongInt;
  i, j: Integer;
begin
  for i := 1 to M do
  begin
    for j := 1 to N do
    begin
      GetItem(i, j, Item);
      Write(F, Item: Format);
    end;
    Writeln(F);
  end;
end;

constructor TSet.Init(ALimit, ADelta: Integer; AItemSize: Byte);
begin
  Limit := aLimit;
  Delta := aDelta;
  ItemSize := AItemSize;
  N := 0;
  Iterate := 0;
  Data := MemAlloc(ItemSize * Limit);
  if Data = nil then Fail;
end;

constructor TSet.Load(var S: TStream);
begin
  S.Read(N, SizeOf(N) + SizeOf(Limit));
  S.Read(Delta, SizeOf(Delta) + SizeOf(ItemSize));
  S.Read(Iterate, SizeOf(Iterate));
  Data := MemAlloc(ItemSize * Limit);
  if Data = nil then Fail;
  S.Read(Data^, ItemSize * N);
end;

procedure TSet.Store(var S: TStream);
begin
  S.Write(N, SizeOf(N) + SizeOf(Limit));
  S.Write(Delta, SizeOf(Delta) + SizeOf(ItemSize));
  S.Write(Iterate, SizeOf(Iterate));
  S.Write(Data^, ItemSize * N);
end;

destructor TSet.Done;
begin
  if Data <> nil then
    FreeMem(Data, ItemSize * Limit);
end;

function TSet.Insert(var Item): ShortInt;
var NewData: PByteArray;
begin
  if N < Limit then
  begin
    Inc(N);
    Move(Item, Data^[(N - 1) * ItemSize + 1], ItemSize);
  end
  else
  begin
    NewData := MemAlloc(ItemSize * (Limit + Delta));
    if NewData = nil then
      Insert := -1
    else
    begin
      Move(Data^, NewData^, ItemSize * N);
      FreeMem(Data, ItemSize * Limit);
      Data := NewData;
      Inc(N);
      Inc(Limit, Delta);
      Move(Item, Data^[(N - 1) * ItemSize + 1], ItemSize);
    end;
  end;
end;

procedure TSet.DisposeAll;
begin
  N := 0;
  Iterate := 0;
end;

procedure TSet.StartIterate;
begin
  Iterate := 0;
end;

function TSet.NextExist: boolean;
begin
  NextExist := (Iterate < N);
end;

procedure TSet.DisposeCurrent;
begin
  Move(Data^[Iterate * ItemSize + 1], Data^[(Iterate - 1) * ItemSize + 1], ItemSize * (N - Iterate));
  Dec(Iterate);
  Dec(N);
end;

procedure TSet.GetNext(var Item);
begin
  Inc(Iterate);
  Move(Data^[(Iterate - 1) * ItemSize + 1], Item, ItemSize);
end;

constructor TIntSet.Init(ALimit, ADelta: Integer);
begin
  inherited Init(ALimit, ADelta, SizeOf(Integer));
end;

function TIntSet.GetNextInteger: Integer;
var Item: Integer;
begin
  GetNext(Item);
  GetNextInteger := Item;
end;

procedure RegisterStrcts;
begin
  RegisterType(RMatrix);
  RegisterType(RCollectionMatrix);
  RegisterType(RLongMatrix);
  RegisterType(RSet);
  RegisterType(RIntSet)
end;

function Print3LongMatrix(var F: Text; A, B, C: PLongMatrix):
  ShortInt;

{      A
     B C      }

function MaxFormat(A: PLongMatrix): Byte;
var
  i, j: Integer;
  Format: Byte;
  S: String;
  Item: LongInt;
begin
  Format := 0;
  with A^ do
  begin
    for i := 1 to M do
      for j := 1 to N do
      begin
        GetItem(i, j, Item);
        Str(Item, S);
        if Length(S) >= Format then Format := Length(S) + 1;
      end;
  end;
  MaxFormat := Format;
end;

var
  Item: LongInt;
  i, j: Integer;
  Format: Byte;
  S: String;
begin
{$I-}
  Print3LongMatrix := -1; {ᯥ譮 믮}
  Format := MaxFormat(A);
  if Format < MaxFormat(B) then Format := MaxFormat(B);
  if Format < MaxFormat(C) then Format := MaxFormat(C);
  write(F, '  ');
  for j := 1 to Format * B^.N do write(F, ' ');
  write(F, ' ');
  for j := 1 to Format * A^.M do write(F, '');
  Writeln(F, 'Ŀ');
  with A^ do
    for i := 1 to N do
    begin
      write(F, '  ');
      for j := 1 to Format * B^.N do write(F, ' ');
      write(F, ' ');
      for j := 1 to M do
      begin
        GetItem(j, i, Item);
        Write(F, Item: Format);
      end;
      Writeln(F, ' ');
    end;
  write(F, ' ');
  for j := 1 to Format * B^.N do write(F, '');
  write(F, '');
  for j := 1 to Format * A^.M do write(F, '');
  Writeln(F, 'Ĵ');
  with B^ do
    for i := 1 to M do
    begin
      write(F, ' ');
      for j := 1 to N do
      begin
        GetItem(i, j, Item);
        Write(F, Item: Format);
      end;
      Write(F, ' ');
      for j := 1 to C^.M do
      begin
        C^.GetItem(j, i, Item);
        Write(F, Item: Format);
      end;
      Writeln(F, ' ');
    end;
  write(F, ' ');
  for j := 1 to Format * B^.N do write(F, '');
  write(F, '');
  for j := 1 to Format * A^.M do write(F, '');
  Writeln(F, '');

{$I+}
  if IOResult = 0 then Print3LongMatrix := 1;
end;


end.
